# SPDX-License-Identifier: GPL-3.0


class Helper(object):
    # noinspection PyUnusedLocal
    def __init__(self, protocol, drm=None):
        """Initialize InputStream Helper class

        :param str protocol:    The protocol to use.
        :param str drm:         The DRM that is required.

        """
        pass

    def check_inputstream(self):
        """ Check if Widevine is installed.

        :return: True if Widevine is installed
        :rtype: bool

        """

        return True
